import {
  StorageStatisticsSegmentDto,
  StorageStatisticsSegmentUnboxedDto
} from "@upload-io/upload-shared-iso/statistics/storage/dtos/shared/StorageStatisticsSegmentDto";
import { UploadStatisticsMimeRecord } from "@upload-io/upload-shared-iso/statistics/storage/dtos/shared/UploadStatisticsMimeRecord";
import {
  StorageStatisticsByTimeDto,
  StorageStatisticsByTimeUnboxedDto,
  CreatesAndDeletesByTimeDto
} from "@upload-io/upload-shared-iso/statistics/storage/dtos/shared/StorageStatisticsByTimeDto";
import {
  StorageStatisticsBySourceDto,
  StorageStatisticsBySourceUnboxedDto
} from "@upload-io/upload-shared-iso/statistics/storage/dtos/shared/StorageStatisticsBySourceDto";

export interface UploadStatisticsDto {
  all: StorageStatisticsSegmentDto;
  byMime: UploadStatisticsMimeRecord<StorageStatisticsSegmentDto>;
  bySource: StorageStatisticsBySourceDto | undefined;
  byTime: StorageStatisticsByTimeDto;
}

export interface UploadStatisticsUnboxedDto {
  all: StorageStatisticsSegmentUnboxedDto;
  byMime: UploadStatisticsMimeRecord<StorageStatisticsSegmentUnboxedDto>;
  bySource: StorageStatisticsBySourceUnboxedDto | null;
  byTime: StorageStatisticsByTimeUnboxedDto;
}

export namespace UploadStatisticsDto {
  export const empty: UploadStatisticsDto = {
    all: StorageStatisticsSegmentDto.empty,
    byMime: {
      audio: StorageStatisticsSegmentDto.empty,
      image: StorageStatisticsSegmentDto.empty,
      video: StorageStatisticsSegmentDto.empty,
      document: StorageStatisticsSegmentDto.empty,
      other: StorageStatisticsSegmentDto.empty
    },
    bySource: StorageStatisticsBySourceDto.empty,
    byTime: CreatesAndDeletesByTimeDto.empty
  };

  export function combine(left: UploadStatisticsDto, right: UploadStatisticsDto): UploadStatisticsDto {
    return {
      all: StorageStatisticsSegmentDto.combine(left.all, right.all),
      byMime: {
        audio: StorageStatisticsSegmentDto.combine(left.byMime.audio, right.byMime.audio),
        image: StorageStatisticsSegmentDto.combine(left.byMime.image, right.byMime.image),
        video: StorageStatisticsSegmentDto.combine(left.byMime.video, right.byMime.video),
        document: StorageStatisticsSegmentDto.combine(left.byMime.document, right.byMime.document),
        other: StorageStatisticsSegmentDto.combine(left.byMime.other, right.byMime.other)
      },
      bySource: StorageStatisticsBySourceDto.combine(
        left.bySource ?? StorageStatisticsBySourceDto.empty,
        right.bySource ?? StorageStatisticsBySourceDto.empty
      ),
      byTime: CreatesAndDeletesByTimeDto.combine(left.byTime, right.byTime)
    };
  }
}
